@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2015-2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2015-2018 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -47,7 +47,7 @@ http://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2015-2017, Damon Lynch"
+__copyright__ = "Copyright 2015-2018, Damon Lynch"
 
 import os
 import sys
@@ -58,6 +58,7 @@ import time
 import shutil
 from collections import namedtuple
 from typing import Optional, Tuple, Union
+import sqlite3
 
 from PyQt5.QtCore import QSize
 from PyQt5.QtGui import QImage
@@ -69,8 +70,7 @@ from raphodo.rpdsql import CacheSQL
 
 
 GetThumbnail = namedtuple('GetThumbnail', 'disk_status, thumbnail, path')
-GetThumbnailPath = namedtuple('GetThumbnailPath', 'disk_status, path, mdatatime, '
-                                                  'orientation_unknown')
+GetThumbnailPath = namedtuple('GetThumbnailPath', 'disk_status, path, mdatatime, orientation_unknown')
 
 class MD5Name:
     """Generate MD5 hashes for file names."""
@@ -439,17 +439,21 @@ class ThumbnailCacheSql:
         else:
             logging.debug("Saving thumbnail for %s in RPD thumbnail cache", uri)
 
-        self.thumb_db.add_thumbnail(uri=uri, size=size, mtime=mtime,
+        try:
+            self.thumb_db.add_thumbnail(uri=uri, size=size, mtime=mtime,
                                     mdatatime=mdatatime,
                                     md5_name=md5_name, orientation_unknown=orientation_unknown,
                                     failure=generation_failed)
+        except sqlite3.OperationalError as e:
+            logging.error("Database error adding thumbnail for %s: %s. Will not retry.", uri, e)
+            return None
+
         if generation_failed:
             return None
 
         md5_full_name = os.path.join(self.cache_dir, md5_name)
 
-        temp_path = os.path.join(self.cache_dir, self.random_filename.name(
-            extension='jpg'))
+        temp_path = os.path.join(self.cache_dir, self.random_filename.name(extension='jpg'))
 
         if thumbnail.save(temp_path, format='jpg', quality=75):
             try:
@@ -521,9 +525,13 @@ class ThumbnailCacheSql:
                     os.remove(thumbnail)
                     deleted_thumbnails.append(name)
             if len(deleted_thumbnails):
-                self.thumb_db.delete_thumbnails(deleted_thumbnails)
-                logging.debug('Deleted {} thumbnail files that had not been '
-                          'accessed for {} or more days'.format(len(deleted_thumbnails), days))
+                if self.thumb_db.cache_exists():
+                    self.thumb_db.delete_thumbnails(deleted_thumbnails)
+                logging.debug(
+                    'Deleted {} thumbnail files that had not been accessed for {} or more days'.format(
+                        len(deleted_thumbnails), days
+                    )
+                )
 
     def purge_cache(self) -> None:
         """
