@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2016 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2015-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -17,7 +17,7 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2015-2016, Damon Lynch"
+__copyright__ = "Copyright 2015-2020, Damon Lynch"
 
 import argparse
 import sys
@@ -27,7 +27,7 @@ import os
 import shlex
 import time
 from collections import deque, namedtuple
-from typing import Optional, Set, List, Dict, Sequence, Any, Tuple
+from typing import Optional, Set, List, Dict, Sequence, Any, Tuple, Union
 
 
 import psutil
@@ -37,10 +37,17 @@ from PyQt5.QtGui import (QPixmap, QImage)
 
 import zmq
 import zmq.log.handlers
-from zmq.eventloop.ioloop import IOLoop
+if zmq.pyzmq_version_info()[0] < 17:
+    from zmq.eventloop import ioloop
+else:
+    try:
+        from tornado import ioloop
+    except ImportError:
+        from zmq.eventloop import ioloop  # note: deprecated in pyzmq 17.0.0
+
 from zmq.eventloop.zmqstream import ZMQStream
 
-from raphodo.rpdfile import (RPDFile, FileTypeCounter, FileSizeSum, Photo, Video)
+from raphodo.rpdfile import RPDFile, FileTypeCounter, FileSizeSum, Photo, Video
 from raphodo.devices import Device
 from raphodo.utilities import CacheDirs, set_pdeathsig
 from raphodo.constants import (
@@ -59,7 +66,7 @@ from raphodo.problemnotification import (
 logger = logging.getLogger()
 
 
-def make_filter_from_worker_id(worker_id) -> bytes:
+def make_filter_from_worker_id(worker_id: Union[int, str]) -> bytes:
     r"""
     Returns a python byte string from an integer or string
 
@@ -75,6 +82,7 @@ def make_filter_from_worker_id(worker_id) -> bytes:
         return worker_id.encode()
     raise(TypeError)
 
+
 def create_identity(worker_type: str, identity: str) -> bytes:
     r"""Generate identity for a worker's 0mq socket.
 
@@ -89,6 +97,7 @@ def create_identity(worker_type: str, identity: str) -> bytes:
     # Replace any whitespace in the strings with a hyphen
     return '{}-{}'.format('-'.join(worker_type.split()), '-'.join(identity.split())).encode()
 
+
 def get_worker_id_from_identity(identity: bytes) -> int:
     r"""Extract worker id from the identity used in a 0mq socket
 
@@ -153,13 +162,15 @@ class ProcessManager:
         self.workers = []  # type: List[int]
 
     def _get_cmd(self) -> str:
-        return '{} {}'.format(sys.executable,
-                              os.path.join(os.path.abspath(os.path.dirname(__file__)),
-                                           self._process_to_run))
+        return '{} {}'.format(
+            sys.executable, os.path.join(
+                os.path.abspath(os.path.dirname(__file__)), self._process_to_run
+            )
+        )
 
     def _get_command_line(self, worker_id: int) -> str:
         """
-        Implement in sublcass
+        Implement in subclass
         """
         return ''
 
@@ -176,8 +187,9 @@ class ProcessManager:
             logging.critical("Failed to start process: %s", command_line)
             logging.critical('OSError [Errno %s]: %s', e.errno, e.strerror)
             if e.errno == 8:
-                logging.critical("Script shebang line might be malformed or missing: %s",
-                                 self._get_cmd())
+                logging.critical(
+                    "Script shebang line might be malformed or missing: %s", self._get_cmd()
+                )
             sys.exit(1)
         logging.debug("Started '%s' with pid %s", command_line, proc.pid)
 
@@ -190,14 +202,19 @@ class ProcessManager:
         Forcefully terminate any running child processes.
         """
 
-
-        zombie_processes = [p for p in self.processes.values()
-                            if p.is_running() and p.status() == psutil.STATUS_ZOMBIE]
-        running_processes = [p for p in self.processes.values()
-                            if p.is_running() and p.status() != psutil.STATUS_ZOMBIE]
+        zombie_processes = [
+            p for p in self.processes.values()
+            if p.is_running() and p.status() == psutil.STATUS_ZOMBIE
+        ]
+        running_processes = [
+            p for p in self.processes.values()
+            if p.is_running() and p.status() != psutil.STATUS_ZOMBIE
+        ]
         if hasattr(self, '_process_name'):
-            logging.debug("Forcefully terminating processes for %s: %s zombies, %s running.",
-                          self._process_name, len(zombie_processes), len(running_processes))
+            logging.debug(
+                "Forcefully terminating processes for %s: %s zombies, %s running.",
+                self._process_name, len(zombie_processes), len(running_processes)
+            )
 
         for p in zombie_processes:  # type: psutil.Process
             try:
@@ -298,7 +315,7 @@ class PullPipelineManager(ProcessManager, QObject):
 
                 if directive == b'cmd':
                     command = content
-                    assert command in [b"STOPPED", b"FINISHED", b"KILL"]
+                    assert command in (b"STOPPED", b"FINISHED", b"KILL")
                     if command == b"KILL":
                         # Terminate immediately, without regard for any
                         # incoming messages. This message is only sent
@@ -365,37 +382,46 @@ class PullPipelineManager(ProcessManager, QObject):
         return [worker_id, b'cmd', b'START']
 
     def start(self) -> None:
-        logging.critical("Member function start() not implemented in child class of %s",
-                         self._process_name)
+        logging.critical(
+            "Member function start() not implemented in child class of %s", self._process_name
+        )
 
     def start_worker(self, worker_id: bytes, data: bytes) -> None:
-        logging.critical("Member function start_worker() not implemented in child class of %s",
-                         self._process_name)
+        logging.critical(
+            "Member function start_worker() not implemented in child class of %s",
+            self._process_name
+        )
 
     def stop(self) -> None:
-        logging.critical("Member function stop() not implemented in child class of %s",
-                         self._process_name)
+        logging.critical(
+            "Member function stop() not implemented in child class of %s", self._process_name
+        )
 
     def stop_worker(self, worker_id: int) -> None:
-        logging.critical("Member function stop_worker() not implemented in child class of %s",
-                         self._process_name)
+        logging.critical(
+            "Member function stop_worker() not implemented in child class of %s",
+            self._process_name
+        )
 
     def pause(self) -> None:
         logging.critical("Member function pause() not implemented in child class of %s",
                          self._process_name)
 
     def resume(self, worker_id: Optional[bytes]) -> None:
-        logging.critical("Member function stop_worker() not implemented in child class of %s",
-                         self._process_name)
+        logging.critical(
+            "Member function stop_worker() not implemented in child class of %s", self._process_name
+        )
 
     def send_message_to_worker(self, data: bytes, worker_id:Optional[bytes]=None) -> None:
         if self.terminating:
-            logging.debug("%s not sending message to worker because manager is terminated",
-                          self._process_name)
+            logging.debug(
+                "%s not sending message to worker because manager is terminated", self._process_name
+            )
             return
         if not self.workers:
-            logging.debug("%s not sending message to worker because there are no workers",
-                          self._process_name)
+            logging.debug(
+                "%s not sending message to worker because there are no workers", self._process_name
+            )
             return
 
         assert isinstance(data, bytes)
@@ -431,16 +457,23 @@ class LoadBalancerWorkerManager(ProcessManager):
         cmd = self._get_cmd()
 
         return '{} --request {} --send {} --identity {} --logging {}'.format(
-                        cmd,
-                        self.backend_port,
-                        self.sink_port,
-                        worker_id,
-                        self.logging_port)
+            cmd,
+            self.backend_port,
+            self.sink_port,
+            worker_id,
+            self.logging_port
+        )
 
     def start_workers(self) -> None:
         for worker_id in range(self.no_workers):
             self.add_worker(worker_id)
 
+    def zombie_workers(self) -> List[int]:
+        return [
+            worker_id for worker_id in self.workers
+            if self.processes[worker_id].status() == psutil.STATUS_ZOMBIE
+        ]
+
 
 class LRUQueue:
     """LRUQueue class using ZMQStream/IOLoop for event dispatching"""
@@ -464,18 +497,18 @@ class LRUQueue:
         self.backend.on_recv(self.handle_backend)
         self.controller.on_recv(self.handle_controller)
 
-        self.loop = IOLoop.instance()
+        self.loop = ioloop.IOLoop.instance()
 
     def handle_controller(self, msg):
         self.terminating = True
-        # logging.debug("%s load balancer requesting %s workers to stop", self.worker_type,
-        #               len(self.workers))
 
         while len(self.workers):
             worker_identity = self.workers.popleft()
 
-            logging.debug("%s load balancer sending stop cmd to worker %s", self.worker_type,
-                          worker_identity.decode())
+            logging.debug(
+                "%s load balancer sending stop cmd to worker %s",
+                self.worker_type, worker_identity.decode()
+            )
             self.backend.send_multipart([worker_identity, b'', b'cmd', b'STOP'])
             self.terminating_workers.add(worker_identity)
 
@@ -488,6 +521,10 @@ class LRUQueue:
         # add worker back to the list of workers
         self.workers.append(worker_identity)
 
+        zw = self.process_manager.zombie_workers()
+        if zw:
+            logging.critical("%s dead thumbnail extractors", len(zw))
+
         # Second frame is empty
         assert empty == b''
 
@@ -501,7 +538,9 @@ class LRUQueue:
                     if p.is_running():
                         pid = p.pid
                         if p.status() != psutil.STATUS_SLEEPING:
-                            logging.debug("Waiting on %s process %s...", p.status(), pid)
+                            logging.debug(
+                                "Waiting on %s process %s...", p.status(), pid
+                            )
                             os.waitpid(pid, 0)
                             logging.debug("...process %s is finished", pid)
                         else:
@@ -571,7 +610,7 @@ class LoadBalancer:
         queue = LRUQueue(backend, frontend, controller, worker_type, process_manager)
 
         # start reactor, which is an infinite loop
-        IOLoop.instance().start()
+        ioloop.IOLoop.instance().start()
 
         # Finished infinite loop: do some housekeeping
         logging.debug("Forcefully terminating load balancer child processes")
@@ -627,11 +666,12 @@ class LoadBalancerManager(ProcessManager, QObject):
         cmd = self._get_cmd()
 
         return '{} --receive {} --send {} --controller {} --logging {}'.format(
-                        cmd,
-                        self.requester_port,
-                        self.sink_port,
-                        self.controller_port,
-                        self.logging_port)
+            cmd,
+            self.requester_port,
+            self.sink_port,
+            self.controller_port,
+            self.logging_port
+        )
 
 DAEMON_WORKER_ID = 0
 
@@ -671,7 +711,8 @@ class PushPullDaemonManager(PullPipelineManager):
             except zmq.Again:
                 logging.debug(
                     "Terminating %s sink because child process did not receive message",
-                    self._process_name)
+                    self._process_name
+                )
                 self.terminate_sink()
         else:
             # The process may have crashed. Stop the sink.
@@ -681,10 +722,11 @@ class PushPullDaemonManager(PullPipelineManager):
         cmd = self._get_cmd()
 
         return '{} --receive {} --send {} --logging {}'.format(
-                        cmd,
-                        self.ventilator_port,
-                        self.receiver_port,
-                        self.logging_port)
+            cmd,
+            self.ventilator_port,
+            self.receiver_port,
+            self.logging_port
+        )
 
     def _get_ventilator_start_message(self, worker_id: int) -> List[bytes]:
         return [b'cmd', b'START']
@@ -786,15 +828,16 @@ class PublishPullPipelineManager(PullPipelineManager):
     def _get_command_line(self, worker_id: int) -> str:
         cmd = self._get_cmd()
 
-        return '{} --receive {} --send {} --controller {} --syncclient {} ' \
-               '--filter {} --logging {}'.format(
-                        cmd,
-                        self.ventilator_port,
-                        self.receiver_port,
-                        self.controller_port,
-                        self.sync_service_port,
-                        worker_id,
-                        self.logging_port)
+        return '{} --receive {} --send {} --controller {} --syncclient {} --filter {} --logging '\
+               '{}'.format(
+            cmd,
+            self.ventilator_port,
+            self.receiver_port,
+            self.controller_port,
+            self.sync_service_port,
+            worker_id,
+            self.logging_port
+        )
 
     def __len__(self) -> int:
         return len(self.workers)
@@ -876,9 +919,9 @@ class WorkerProcess():
 
         if self.worker_id is not None:
             name = '{}-{}'.format(name, self.worker_id.decode())
-        self.logger_publisher = ProcessLoggerPublisher(context=self.context,
-                                                       name=name,
-                                                       notification_port=notification_port)
+        self.logger_publisher = ProcessLoggerPublisher(
+            context=self.context, name=name, notification_port=notification_port
+        )
 
     def send_message_to_sink(self) -> None:
 
@@ -941,15 +984,17 @@ class DaemonProcess(WorkerProcess):
             assert content == b'STOP'
             self.cleanup_pre_stop()
             # signal to sink that we've terminated before finishing
-            self.sender.send_multipart([make_filter_from_worker_id(
-                DAEMON_WORKER_ID), b'cmd', b'STOPPED'])
+            self.sender.send_multipart(
+                [make_filter_from_worker_id(DAEMON_WORKER_ID), b'cmd', b'STOPPED']
+            )
             sys.exit(0)
 
     def send_message_to_sink(self) -> None:
         # Must use a dummy value for the worker id, as there is only ever one
         # instance.
-        self.sender.send_multipart([make_filter_from_worker_id(
-            DAEMON_WORKER_ID), b'data', self.content])
+        self.sender.send_multipart(
+            [make_filter_from_worker_id(DAEMON_WORKER_ID), b'data', self.content]
+        )
 
 
 class WorkerInPublishPullPipeline(WorkerProcess):
@@ -1016,7 +1061,7 @@ class WorkerInPublishPullPipeline(WorkerProcess):
             # If there is no command,exception will occur
             worker_id, command = self.controller.recv_multipart(zmq.DONTWAIT)
             assert command in [b'PAUSE', b'STOP']
-            assert  worker_id == self.worker_id
+            assert worker_id == self.worker_id
 
             if command == b'PAUSE':
                 # Because the process is paused, do a blocking read to
@@ -1071,9 +1116,9 @@ class LoadBalancerWorker:
         self.sender = self.context.socket(zmq.PUSH)
         self.sender.connect("tcp://localhost:{}".format(args.send))
         
-        self.logger_publisher = ProcessLoggerPublisher(context=self.context,
-                                                       name=worker_type,
-                                                       notification_port=args.logging)
+        self.logger_publisher = ProcessLoggerPublisher(
+            context=self.context, name=worker_type, notification_port=args.logging
+        )
 
         # Tell the load balancer we are ready for work
         self.requester.send(b"READY")
@@ -1204,10 +1249,6 @@ class ScanArguments:
         :param ignore_other_types: ignore file types like TIFF
         :param log_gphoto2: whether to generate detailed gphoto2 log
          messages
-        :param scan_only_DCIM: if the device is an auto-detected volume,
-         then if True, scan only in it's DCIM folder
-        :param warn_unknown_file: whether to issue a warning when
-         encountering an unknown (unrecognized) file
         """
 
         self.device = device
@@ -1232,7 +1273,9 @@ class ScanResults:
                  sample_video: Optional[Video]=None,
                  problems: Optional[ScanProblems]=None,
                  fatal_error: Optional[bool]=None,
-                 entire_video_required: Optional[bool]=None) -> None:
+                 camera_removed: Optional[bool]=None,
+                 entire_video_required: Optional[bool]=None,
+                 entire_photo_required: Optional[bool]=None) -> None:
         self.rpd_files = rpd_files
         self.file_type_counter = file_type_counter
         self.file_size_sum = file_size_sum
@@ -1245,7 +1288,9 @@ class ScanResults:
         self.sample_video = sample_video
         self.problems = problems
         self.fatal_error = fatal_error
+        self.camera_removed = camera_removed
         self.entire_video_required = entire_video_required
+        self.entire_photo_required = entire_photo_required
 
 
 class CopyFilesArguments:
@@ -1285,7 +1330,8 @@ class CopyFilesResults:
                  rpd_file: Optional[RPDFile]=None,
                  download_count: Optional[int]=None,
                  mdata_exceptions: Optional[Tuple]=None,
-                 problems: Optional[CopyingProblems]=None) -> None:
+                 problems: Optional[CopyingProblems]=None,
+                 camera_removed: Optional[bool]=None) -> None:
         """
 
         :param scan_id: scan id of the device the files are being
@@ -1320,7 +1366,7 @@ class CopyFilesResults:
         self.download_count = download_count
         self.mdata_exceptions = mdata_exceptions
         self.problems = problems
-
+        self.camera_removed = camera_removed
 
 
 class ThumbnailDaemonData:
@@ -1336,13 +1382,15 @@ class ThumbnailDaemonData:
                  write_fdo_thumbnail: Optional[bool]=None,
                  use_thumbnail_cache: Optional[bool]=None,
                  backup_full_file_names: Optional[List[str]]=None,
-                 fdo_name: Optional[str]=None) -> None:
+                 fdo_name: Optional[str]=None,
+                 force_exiftool: Optional[bool]=None) -> None:
         self.frontend_port = frontend_port
         self.rpd_file = rpd_file
         self.write_fdo_thumbnail = write_fdo_thumbnail
         self.use_thumbnail_cache = use_thumbnail_cache
         self.backup_full_file_names = backup_full_file_names
         self.fdo_name = fdo_name
+        self.force_exiftool = force_exiftool
 
 
 class RenameAndMoveFileData:
@@ -1457,12 +1505,14 @@ class GenerateThumbnailsArguments:
                  name: str,
                  proximity_seconds: int,
                  cache_dirs: CacheDirs,
+                 need_photo_cache_dir: bool,
                  need_video_cache_dir: bool,
                  frontend_port: int,
                  log_gphoto2: bool,
                  camera: Optional[str]=None,
                  port: Optional[str]=None,
-                 entire_video_required: Optional[bool]=None) -> None:
+                 entire_video_required: Optional[bool]=None,
+                 entire_photo_required: Optional[bool]=None) -> None:
         """
         List of files for which thumbnails are to be generated.
         All files  are assumed to have the same scan id.
@@ -1474,6 +1524,8 @@ class GenerateThumbnailsArguments:
          generation
         :param cache_dirs: the location where the cache directories
          should be created
+        :param need_photo_cache_dir: if True, must use cache dir
+         to extract photo thumbnail
         :param need_video_cache_dir: if True, must use cache dir
          to extract video thumbnail
         :param frontend_port: port to use to send to load balancer's
@@ -1483,8 +1535,10 @@ class GenerateThumbnailsArguments:
          camera, this is the name of the camera, else None
         :param port: If the thumbnails are being downloaded from a
          camera, this is the port of the camera, else None
-        :param entire_video_required:  if the entire video is
-         required to extract the thumbnail
+        :param entire_video_required: if the entire video is required
+         to extract the thumbnail
+        :param entire_photo_required: if the entire photo is required
+         to extract the thumbnail
         """
 
         self.rpd_files = rpd_files
@@ -1492,6 +1546,7 @@ class GenerateThumbnailsArguments:
         self.name = name
         self.proximity_seconds = proximity_seconds
         self.cache_dirs = cache_dirs
+        self.need_photo_cache_dir = need_photo_cache_dir
         self.need_video_cache_dir = need_video_cache_dir
         self.frontend_port = frontend_port
         if camera is not None:
@@ -1501,18 +1556,21 @@ class GenerateThumbnailsArguments:
         self.port = port
         self.log_gphoto2 = log_gphoto2
         self.entire_video_required = entire_video_required
+        self.entire_photo_required = entire_photo_required
 
 
 class GenerateThumbnailsResults:
     def __init__(self, rpd_file: Optional[RPDFile]=None,
                  thumbnail_bytes: Optional[bytes]=None,
                  scan_id: Optional[int]=None,
-                 cache_dirs: Optional[CacheDirs]=None) -> None:
+                 cache_dirs: Optional[CacheDirs]=None,
+                 camera_removed: Optional[bool]=None,) -> None:
         self.rpd_file = rpd_file
         # If thumbnail_bytes is None, there is no thumbnail
         self.thumbnail_bytes = thumbnail_bytes
         self.scan_id = scan_id
         self.cache_dirs = cache_dirs
+        self.camera_removed = camera_removed
 
 
 class ThumbnailExtractorArgument:
@@ -1526,7 +1584,8 @@ class ThumbnailExtractorArgument:
                  use_thumbnail_cache: bool,
                  file_to_work_on_is_temporary: bool,
                  write_fdo_thumbnail: bool,
-                 send_thumb_to_main: bool) -> None:
+                 send_thumb_to_main: bool,
+                 force_exiftool: bool) -> None:
         self.rpd_file = rpd_file
         self.task = task
         self.processing = processing
@@ -1538,6 +1597,7 @@ class ThumbnailExtractorArgument:
         self.use_thumbnail_cache = use_thumbnail_cache
         self.write_fdo_thumbnail = write_fdo_thumbnail
         self.send_thumb_to_main = send_thumb_to_main
+        self.force_exiftool = force_exiftool
 
 
 class RenameMoveFileManager(PushPullDaemonManager):
@@ -1567,8 +1627,7 @@ class RenameMoveFileManager(PushPullDaemonManager):
             assert data.stored_sequence_no is not None
             assert data.downloads_today is not None
             assert isinstance(data.downloads_today, list)
-            self.sequencesUpdate.emit(data.stored_sequence_no,
-                                      data.downloads_today)
+            self.sequencesUpdate.emit(data.stored_sequence_no, data.downloads_today)
 
 
 class ThumbnailDaemonManager(PushPullDaemonManager):
@@ -1627,12 +1686,13 @@ class ScanManager(PublishPullPipelineManager):
     this computer path)
     """
     scannedFiles = pyqtSignal(
-        'PyQt_PyObject', 'PyQt_PyObject', FileTypeCounter, 'PyQt_PyObject', bool
+        'PyQt_PyObject', 'PyQt_PyObject', FileTypeCounter, 'PyQt_PyObject', bool, bool
     )
     deviceError = pyqtSignal(int, CameraErrorCode)
     deviceDetails = pyqtSignal(int, 'PyQt_PyObject', 'PyQt_PyObject', str)
     scanProblems = pyqtSignal(int, 'PyQt_PyObject')
     fatalError = pyqtSignal(int)
+    cameraRemovedDuringScan = pyqtSignal(int)
 
     def __init__(self, logging_port: int) -> None:
         super().__init__(logging_port=logging_port, thread_name=ThreadNames.scan)
@@ -1645,12 +1705,14 @@ class ScanManager(PublishPullPipelineManager):
             assert data.file_type_counter
             assert data.file_size_sum
             assert data.entire_video_required is not None
+            assert  data.entire_photo_required is not None
             self.scannedFiles.emit(
                 data.rpd_files,
                 (data.sample_photo, data.sample_video),
                 data.file_type_counter,
                 data.file_size_sum,
-                data.entire_video_required
+                data.entire_video_required,
+                data.entire_photo_required
             )
         else:
             assert data.scan_id is not None
@@ -1663,6 +1725,10 @@ class ScanManager(PublishPullPipelineManager):
                 )
             elif data.problems is not None:
                 self.scanProblems.emit(data.scan_id, data.problems)
+            elif data.camera_removed is not None:
+                self.cameraRemovedDuringScan.emit(
+                    data.scan_id
+                )
             else:
                 assert data.fatal_error
                 self.fatalError.emit(data.scan_id)
@@ -1699,7 +1765,8 @@ class BackupManager(PublishPullPipelineManager):
             assert data.rpd_file is not None
             self.message.emit(
                 data.device_id, data.backup_succeeded, data.do_backup, data.rpd_file,
-                data.backup_full_file_name, data.mdata_exceptions)
+                data.backup_full_file_name, data.mdata_exceptions
+            )
         else:
             assert data.problems is not None
             self.backupProblems.emit(data.device_id, data.problems)
@@ -1715,6 +1782,7 @@ class CopyFilesManager(PublishPullPipelineManager):
     tempDirs = pyqtSignal(int, str,str)
     bytesDownloaded = pyqtSignal(int, 'PyQt_PyObject', 'PyQt_PyObject')
     copyProblems = pyqtSignal(int, 'PyQt_PyObject')
+    cameraRemoved = pyqtSignal(int)
 
     def __init__(self, logging_port: int) -> None:
         super().__init__(logging_port=logging_port, thread_name=ThreadNames.copy)
@@ -1735,11 +1803,14 @@ class CopyFilesManager(PublishPullPipelineManager):
         elif data.copy_succeeded is not None:
             assert data.rpd_file is not None
             assert data.download_count is not None
-            self.message.emit(data.copy_succeeded, data.rpd_file, data.download_count,
-                              data.mdata_exceptions)
+            self.message.emit(
+                data.copy_succeeded, data.rpd_file, data.download_count, data.mdata_exceptions
+            )
 
         elif data.problems is not None:
             self.copyProblems.emit(data.scan_id, data.problems)
+        elif data.camera_removed is not None:
+            self.cameraRemoved.emit(data.scan_id)
 
         else:
             assert (data.photo_temp_dir is not None and
