@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2017 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2015-2020 Damon Lynch <damonlynch@gmail.com>
 # Copyright (C) 2008-2015 Canonical Ltd.
 # Copyright (C) 2013 Bernard Baeyens
 
@@ -43,7 +43,7 @@ regarding mount points and XDG related functionality.
 """
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2017, Damon Lynch. Copyright 2008-2015 Canonical Ltd. Copyright" \
+__copyright__ = "Copyright 2011-2020, Damon Lynch. Copyright 2008-2015 Canonical Ltd. Copyright" \
                 " 2013 Bernard Baeyens."
 
 import logging
@@ -56,7 +56,7 @@ import shlex
 import pwd
 import shutil
 from collections import namedtuple
-from typing import Optional, Tuple, List, Dict, Any
+from typing import Optional, Tuple, List, Dict, Set
 from urllib.request import pathname2url, quote
 from tempfile import NamedTemporaryFile
 
@@ -73,9 +73,11 @@ gi.require_version('GExiv2', '0.10')
 gi.require_version('GLib', '2.0')
 from gi.repository import GUdev, UDisks, GLib
 
-from gettext import gettext as _
 
-from raphodo.constants import Desktop, Distro, FileManagerType
+
+from raphodo.constants import (
+    Desktop, Distro, FileManagerType, DefaultFileBrowserFallback, FileManagerBehavior
+)
 from raphodo.utilities import (
     process_running, log_os_release, remove_topmost_directory_from_path, find_mount_point
 )
@@ -106,9 +108,25 @@ def get_distro_id(id_or_id_like: str) -> Distro:
 
 
 def get_distro() -> Distro:
-    if os.path.isfile('/etc/os-release'):
-        with open('/etc/os-release', 'r') as f:
+    """
+    Determine the Linux distribution using /etc/os-release
+    """
+
+    os_release = '/etc/os-release'
+    if os.path.isfile(os_release):
+        with open(os_release, 'r') as f:
             for line in f:
+                if line.startswith('NAME='):
+                    if line.find('elementary') > 0:
+                        return Distro.elementary
+                    if line.find('CentOS Linux') > 0:
+                        return Distro.centos
+                    if line.find('openSUSE') > 0:
+                        return Distro.opensuse
+                    if line.find('Deepin') > 0:
+                        return Distro.deepin
+                    if line.find('Zorin') > 0:
+                        return Distro.zorin
                 if line.startswith('ID='):
                     return get_distro_id(line[3:])
                 if line.startswith('ID_LIKE='):
@@ -154,20 +172,22 @@ def get_media_dir() -> str:
 
     if sys.platform.startswith('linux'):
         media_dir = '/media/{}'.format(get_user_name())
-        run_media_dir = '/run{}'.format(media_dir)
+        run_media_dir = '/run/media'
         distro = get_distro()
         if os.path.isdir(run_media_dir) and distro not in (
                 Distro.ubuntu, Distro.debian, Distro.neon, Distro.galliumos, Distro.peppermint,
-                Distro.elementary):
+                Distro.elementary, Distro.zorin):
             if distro not in (Distro.fedora, Distro.manjaro, Distro.arch, Distro.opensuse,
-                              Distro.gentoo, Distro.antergos):
-                logging.debug("Detected /run/media directory, but distro does not appear to "
-                              "be Fedora, Arch, openSUSE, Gentoo, Manjaro or Antergos")
+                              Distro.gentoo, Distro.antergos, Distro.centos):
+                logging.debug(
+                    "Detected /run/media directory, but distro does not appear to be CentOS, "
+                    "Fedora, Arch, openSUSE, Gentoo, Korora, Manjaro, or Antergos"
+                )
                 log_os_release()
             return run_media_dir
         return media_dir
     else:
-        raise ("Mounts.setValidMountPoints() not implemented on %s", sys.platform())
+        raise ("Mounts.setValidMountPoints() not implemented on %s", sys.platform)
 
 
 _gvfs_gphoto2 = re.compile('gvfs.*gphoto2.*host')
@@ -379,6 +399,13 @@ def get_desktop() -> Desktop:
         env = 'ubuntugnome'
     elif env == 'pop:gnome':
         env = 'popgnome'
+    elif env == 'gnome-classic:gnome':
+        env = 'gnome'
+    elif env == 'budgie:gnome':
+        env = 'gnome'
+    elif env == 'zorin:gnome':
+        env = 'zorin'
+
     try:
         return Desktop[env]
     except KeyError:
@@ -400,7 +427,8 @@ def gvfs_controls_mounts() -> bool:
 
     desktop = get_desktop()
     if desktop in (Desktop.gnome, Desktop.unity, Desktop.cinnamon, Desktop.xfce,
-                   Desktop.mate, Desktop.lxde):
+                   Desktop.mate, Desktop.lxde, Desktop.ubuntugnome,
+                   Desktop.popgnome, Desktop.gnome, Desktop.lxqt):
         return True
     elif desktop == Desktop.kde:
         return False
@@ -566,20 +594,68 @@ def get_fdo_cache_thumb_base_directory() -> str:
     return os.path.join(BaseDirectory.xdg_cache_home, 'thumbnails')
 
 
-def get_default_file_manager(remove_args: bool = True) -> Tuple[
-                                                        Optional[str], Optional[FileManagerType]]:
+# Module level variables important for determining among other things the generation of URIs
+# Pretty ugly, but the alternative is passing values around between several processes
+_desktop = get_desktop()
+_quoted_comma = quote(',')
+_default_file_manager_probed = False
+_default_file_manager = None
+_default_file_manager_type = None
+
+
+def _default_file_manager_for_desktop() -> Tuple[Optional[str], Optional[FileManagerType]]:
+    """
+    If default file manager cannot be determined using system tools, guess
+    based on desktop environment.
+
+    Sets module level globals if found.
+
+    :return: file manager command (without path), and type; if not detected, (None, None)
+    """
+
+    global _default_file_manager
+    global _default_file_manager_type
+
+    try:
+        fm = ''
+        fm = DefaultFileBrowserFallback[_desktop.name]
+        assert shutil.which(fm)
+        t = FileManagerBehavior[fm]
+        _default_file_manager = fm
+        _default_file_manager_type = t
+        return fm, t
+    except KeyError:
+        logging.debug("Error determining default file manager")
+        return None, None
+    except AssertionError:
+        logging.debug("Default file manager %s cannot be found", fm)
+        return None, None
+
+
+def get_default_file_manager() -> Tuple[Optional[str], Optional[FileManagerType]]:
     """
     Attempt to determine the default file manager for the system
     :param remove_args: if True, remove any arguments such as %U from
      the returned command
-    :return: command (without path) if found, else None
+    :return: file manager command (without path), and type; if not detected, (None, None)
     """
+
+    global _default_file_manager_probed
+    global _default_file_manager
+    global _default_file_manager_type
+
+    if _default_file_manager_probed:
+        return _default_file_manager, _default_file_manager_type
+
+    _default_file_manager_probed = True
+
     assert sys.platform.startswith('linux')
     cmd = shlex.split('xdg-mime query default inode/directory')
     try:
         desktop_file = subprocess.check_output(cmd, universal_newlines=True)  # type: str
     except:
-        return None, None
+        return _default_file_manager_for_desktop()
+
     # Remove new line character from output
     desktop_file = desktop_file[:-1]
     if desktop_file.endswith(';'):
@@ -591,46 +667,52 @@ def get_default_file_manager(remove_args: bool = True) -> Tuple[
             try:
                 desktop_entry = DesktopEntry(path)
             except xdg.Exceptions.ParsingError:
-                return None, None
+                return _default_file_manager_for_desktop()
             try:
                 desktop_entry.parse(path)
             except:
-                return None, None
+                return _default_file_manager_for_desktop()
+
             fm = desktop_entry.getExec()
-            if fm.startswith('dolphin'):
-                file_manager_type = FileManagerType.select
-            else:
+
+            # Unhelpful results
+            if fm.startswith('baobab') or fm.startswith('exo-open'):
+                logging.debug('%s returned as default file manager: will substitute', fm)
+                return _default_file_manager_for_desktop()
+
+            # Strip away any extraneous arguments
+            fm_cmd = fm.split()[0]
+            # Strip away any path information
+            fm_cmd = os.path.split(fm_cmd)[1]
+            try:
+                file_manager_type = FileManagerBehavior[fm_cmd]
+            except KeyError:
                 file_manager_type = FileManagerType.regular
-            if remove_args:
-                return fm.split()[0], file_manager_type
-            else:
-                return fm, file_manager_type
 
-    # Special case: LXQt
-    if get_desktop() == Desktop.lxqt:
-        if shutil.which('pcmanfm-qt'):
-            return 'pcmanfm-qt', FileManagerType.regular
+            _default_file_manager = fm_cmd
+            _default_file_manager_type = file_manager_type
+            return _default_file_manager, file_manager_type
+
+    # Special case: no base dirs set, e.g. LXQt
+    return _default_file_manager_for_desktop()
 
-    return None, None
 
 def open_in_file_manager(file_manager: str,
                          file_manager_type: FileManagerType,
                          uri: str) -> None:
-    if file_manager_type == FileManagerType.regular:
-        arg = ''
-    else:
+    if file_manager_type == FileManagerType.select:
         arg = '--select '
+    elif file_manager_type == FileManagerType.show_item:
+        arg = '--show-item '
+    else:
+        arg = ''
 
-    cmd = '{} {}"{}"'.format(file_manager, arg, uri)
+    cmd = '{} {}{}'.format(file_manager, arg, uri)
     logging.debug("Launching: %s", cmd)
     args = shlex.split(cmd)
     subprocess.Popen(args)
 
 
-_desktop = get_desktop()
-_quoted_comma = quote(',')
-
-
 def get_uri(full_file_name: Optional[str]=None,
             path: Optional[str]=None,
             camera_details: Optional[CameraDetails]=None,
@@ -643,16 +725,19 @@ def get_uri(full_file_name: Optional[str]=None,
     :param path: straight path when not passing a full_file_name
     :param camera_details: see named tuple CameraDetails for parameters
     :param desktop_environment: if True, will to generate a URI accepted
-     by Gnome and KDE desktops, which means adjusting the URI if it appears to be an
-     MTP mount. Includes the port too.
+     by Gnome, KDE and other desktops, which means adjusting the URI if it appears to be an
+     MTP mount. Includes the port too, for cameras. Takes into account
+     file manager characteristics.
     :return: the URI
     """
 
+    if not _default_file_manager_probed:
+        get_default_file_manager()
+
     if camera_details is None:
         prefix = 'file://'
         if desktop_environment:
-            desktop = get_desktop()
-            if full_file_name and desktop == Desktop.mate:
+            if full_file_name and _default_file_manager_type == FileManagerType.dir_only_uri:
                 full_file_name = os.path.dirname(full_file_name)
     else:
         if not desktop_environment:
@@ -682,13 +767,14 @@ def get_uri(full_file_name: Optional[str]=None,
             else:
                 prefix = 'gphoto2://' + pathname2url('[{}]'.format(camera_details.port))
 
-            if _desktop == Desktop.lxqt:
+            if _default_file_manager == 'pcmanfm-qt':
                 # pcmanfm-qt does not like the quoted form of the comma
                 prefix = prefix.replace(_quoted_comma, ',')
                 if full_file_name:
                     # pcmanfm-qt does not like the the filename as part of the path
                     full_file_name = os.path.dirname(full_file_name)
 
+
     if full_file_name or path:
         uri = '{}{}'.format(prefix, pathname2url(full_file_name or path))
     else:
@@ -729,8 +815,10 @@ def validate_download_folder(path: Optional[str],
                 # the path is in fact writeable -- can happen with NFS
                 valid = True
         except Exception:
-            logging.warning('While validating download / backup folder, failed to write a '
-                            'temporary file to %s', path)
+            logging.warning(
+                'While validating download / backup folder, failed to write a temporary file to '
+                '%s', path
+            )
 
     return ValidatedFolder(valid, absolute_path)
 
@@ -773,7 +861,8 @@ def udev_attributes(devname: str) -> Optional[UdevAttr]:
     for device in enumerator.execute():
         model = device.get_property('ID_MODEL')  # type: str
         if model is not None:
-            is_mtp = device.get_property('ID_MTP_DEVICE') == '1'
+            is_mtp = device.get_property('ID_MTP_DEVICE') == '1' or \
+                     device.get_property('ID_MEDIA_PLAYER') == '1'
             vendor = device.get_property('ID_VENDOR')  # type: str
             model = model.replace('_', ' ').strip()
             vendor = vendor.replace('_', ' ').strip()
@@ -860,9 +949,13 @@ class CameraHotplug(QObject):
         logging.debug("... camera hotplug monitor started")
         self.enumerateCameras()
         if self.cameras:
-            logging.debug("Camera Hotplug found %d cameras:", len(self.cameras))
+            logging.info(
+                "Camera Hotplug found %d camera(s): %s", len(self.cameras), ', '.join(
+                    (model for port, model in self.cameras.items())
+                )
+            )
             for port, model in self.cameras.items():
-                logging.debug("%s at %s", model, port)
+                logging.debug("%s is at %s", model, port)
 
     def enumerateCameras(self):
         """
@@ -892,34 +985,43 @@ class CameraHotplug(QObject):
         path = device.get_sysfs_path()
         parent_device = device.get_parent()
         parent_path = parent_device.get_sysfs_path()
-        logging.debug("Device change: %s. Path: %s Parent Device: %s Parent path: %s",
-                      action, path, parent_device, parent_path)
+        logging.debug("Device change: %s. Path: %s Parent path: %s", action, path, parent_path)
+
+        # Ignore 'bind' action: seems to add nothing we need to know
 
         if action == 'add':
             if parent_path not in self.cameras:
                 model = device.get_property('ID_MODEL')
-                logging.debug("Hotplug: new camera: %s", model)
+                logging.info("Hotplug: new camera: %s", model.replace('_', ' '))
                 self.cameras[path] = model
                 self.cameraAdded.emit()
             else:
-                logging.debug("Hotplug: already know about %s", self.cameras[
-                    parent_path])
+                logging.debug("Hotplug: already know about %s", self.cameras[parent_path])
 
         elif action == 'remove':
             emit_remove = False
             name = ''
-            if path in self.cameras:
-                name = self.cameras[path]
-                del self.cameras[path]
-                emit_remove = True
-            elif device.get_property('ID_GPHOTO2') == '1':
-                # This should not need to be called. However,
-                # self.enumerateCameras may not have been called earlier
-                name = device.get_property('ID_MODEL')
-                if name is not None:
+
+            # A path might look like:
+            # /sys/devices/pci0000:00/0000:00:1c.6/0000:0e:00.0/usb3/3-2/3-2:1.0
+            # When what we want is:
+            # /sys/devices/pci0000:00/0000:00:1c.6/0000:0e:00.0/usb3/3-2
+            # This unchanged path used to work, so test both the unchanged and modified
+            # path
+            # Note enumerateCameras() above finds only the path as in the 2nd type, without the
+            # 3-2:1.0
+            split_path = os.path.split(path)[0]
+
+            for p in (path, split_path):
+                if p in self.cameras:
+                    name = self.cameras[p]
+                    logging.debug("Hotplug: removing %s on basis of path %s", name, p)
+                    del self.cameras[p]
                     emit_remove = True
+                    break
+
             if emit_remove:
-                logging.debug("Hotplug: %s has been removed", name)
+                logging.info("Hotplug: %s has been removed", name)
                 self.cameraRemoved.emit()
 
 
@@ -1195,6 +1297,7 @@ if have_gio:
         partitionUnmounted = pyqtSignal(str)
         volumeAddedNoAutomount = pyqtSignal()
         cameraPossiblyRemoved = pyqtSignal()
+        cameraVolumeAdded = pyqtSignal(str)
 
         def __init__(self, validMounts: ValidMounts) -> None:
             super().__init__()
@@ -1205,7 +1308,17 @@ if have_gio:
             self.vm.connect('volume-removed', self.volumeRemoved)
             self.portSearch = re.compile(r'usb:([\d]+),([\d]+)')
             self.scsiPortSearch = re.compile(r'usbscsi:(.+)')
+            self.possibleCamera = re.compile(r'/usb/([\d]+)/([\d]+)')
             self.validMounts = validMounts
+            self.camera_volumes_added = dict()  # type: Dict[str, str]
+            self.camera_volumes_mounted = set()  # type: Set[str]
+
+        def mountMightBeCamera(self, mount: Gio.Mount) -> bool:
+            """
+            :param mount: the mount to check
+            :return: True if the mount needs to be checked if it is a camera
+            """
+            return not mount.is_shadowed() and mount.get_volume() is not None
 
         def ptpCameraMountPoint(self, model: str, port: str) -> Optional[Gio.Mount]:
             """
@@ -1218,19 +1331,29 @@ if have_gio:
             if p is not None:
                 p1 = p.group(1)
                 p2 = p.group(2)
-                pattern = re.compile(r'%\S\Susb%\S\S{}%\S\S{}%\S\S'.format(p1, p2))
+                device_path = '/dev/bus/usb/{}/{}'.format(p1, p2)
+                return self.cameraMountPointByUnixDevice(device_path=device_path)
             else:
                 p = self.scsiPortSearch.match(port)
                 if p is None:
                     logging.error("Unknown camera mount method %s %s", model, port)
                 return None
 
+        def cameraMountPointByUnixDevice(self, device_path: str) -> Optional[Gio.Mount]:
+            """
+            :return: the mount point of the PTP / MTP camera, if it is mounted,
+             else None. If camera is not mounted with PTP / MTP, None is
+             returned.
+            """
+
             to_unmount = None
 
             for mount in self.vm.get_mounts():
-                folder_extract = self.mountIsCamera(mount)
-                if folder_extract is not None:
-                    if pattern.match(folder_extract):
+                if self.mountMightBeCamera(mount):
+                    identifier = mount.get_volume().get_identifier(
+                        Gio.VOLUME_IDENTIFIER_KIND_UNIX_DEVICE
+                    )
+                    if device_path == identifier:
                         to_unmount = mount
                         break
             return to_unmount
@@ -1363,28 +1486,33 @@ if have_gio:
                 logging.error('Exception occurred unmounting %s', path)
                 logging.exception('Traceback:')
 
+        def mountIsCamera(self, mount: Gio.Mount) -> bool:
+            """
+            Determine if the mount refers to a camera by checking the
+            path to see if gphoto2 or mtp is in the last folder in the
+            root path
 
-        def mountIsCamera(self, mount: Gio.Mount) -> Optional[str]:
+            :param mount: mount to check
+            :return: True if mount refers to a camera, else False
             """
-            Determine if the mount point is that of a camera
-            :param mount: the mount to examine
-            :return: None if not a camera, or the component of the
-            folder name that indicates on which port it is mounted
-            """
-            root = mount.get_root()
-            if root is None:
-                logging.warning('Unable to get mount root')
-            else:
-                path = root.get_path()
-                if path:
-                    logging.debug("GIO: Looking for camera at mount {}".format(path))
-                    folder_name = os.path.split(path)[1]
-                    for s in ('gphoto2:host=', 'mtp:host='):
-                        if folder_name.startswith(s):
-                            return folder_name[len(s):]
-                if path is not None:
-                    logging.debug("GIO: camera not found at {}".format(path))
-            return None
+
+            if self.mountMightBeCamera(mount):
+                root = mount.get_root()
+                if root is None:
+                    logging.warning('Unable to get mount root')
+                else:
+                    path = root.get_path()
+                    if path:
+                        logging.debug("GIO: Looking for camera at mount {}".format(path))
+                        # check last two levels of the path name, as it might be in a format like
+                        # /run/..../gvfs/gphoto2:host=Canon_Inc._Canon_Digital_Camera/store_00010001
+                        for i in (1, 2):
+                            path, folder_name = os.path.split(path)
+                            if folder_name:
+                                for s in ('gphoto2:host=', 'mtp:host='):
+                                    if folder_name.startswith(s):
+                                        return True
+            return False
 
         def mountIsPartition(self, mount: Gio.Mount) -> bool:
             """
@@ -1409,13 +1537,25 @@ if have_gio:
             return False
 
         def mountAdded(self, volumeMonitor, mount: Gio.Mount) -> None:
+            logging.debug("Examining mount %s", mount.get_name())
+            try:
+                identifier = mount.get_volume().get_identifier(
+                    Gio.VOLUME_IDENTIFIER_KIND_UNIX_DEVICE
+                )
+                if identifier in self.camera_volumes_added:
+                    logging.debug("%s is now mounted", self.camera_volumes_added[identifier])
+                    self.camera_volumes_mounted.add(identifier)
+                    self.cameraMounted.emit()
+                    return
+            except Exception:
+                pass
             if self.mountIsCamera(mount):
                 self.cameraMounted.emit()
             elif self.mountIsPartition(mount):
                 icon_names = self.getIconNames(mount)
-                self.partitionMounted.emit(mount.get_root().get_path(),
-                                           icon_names,
-                                           mount.can_eject())
+                self.partitionMounted.emit(
+                    mount.get_root().get_path(), icon_names, mount.can_eject()
+                )
 
         def mountRemoved(self, volumeMonitor, mount: Gio.Mount) -> None:
             if not self.mountIsCamera(mount):
@@ -1424,12 +1564,43 @@ if have_gio:
                     self.partitionUnmounted.emit(mount.get_root().get_path())
 
         def volumeAdded(self, volumeMonitor, volume: Gio.Volume) -> None:
-            logging.debug("GIO: Volume added %s. Automount: %s",
-                          volume.get_name(),
-                          volume.should_automount())
-            if not volume.should_automount():
-                # TODO is it possible to determine the device type?
-                self.volumeAddedNoAutomount.emit()
+            volume_name = volume.get_name()
+            logging.debug(
+                "GIO: Volume added %s. Automount: %s (might be incorrect)",
+                volume_name, volume.should_automount()
+            )
+
+            # Even if volume.should_automount(), the volume in fact may not be mounted
+            # automatically. Unbelievable.
+
+            device_path = volume.get_identifier(
+                Gio.VOLUME_IDENTIFIER_KIND_UNIX_DEVICE
+            )
+
+            camera_added = self.possibleCamera.search(device_path) is not None
+            if camera_added:
+                self.camera_volumes_added[device_path] = volume_name
+                logging.debug(
+                    "Assuming %s could be a camera at %s", volume_name, device_path
+                )
+                # Time is in milliseconds; 3000 is 3 seconds.
+                QTimer.singleShot(3000, lambda: self.cameraVolumeAddedCheckMount(device_path))
+
+            # if not volume.should_automount():
+            #     # TODO is it possible to determine the device type?
+            #     self.volumeAddedNoAutomount.emit()
+
+        def cameraVolumeAddedCheckMount(self, device_path) -> None:
+            if device_path not in self.camera_volumes_mounted:
+                logging.debug(
+                    "%s had not been automatically mounted. Will initiate camera scan.",
+                    self.camera_volumes_added[device_path]
+                )
+                self.cameraVolumeAdded.emit(device_path)
+            else:
+                logging.debug(
+                    "%s had been automatically mounted", self.camera_volumes_added[device_path]
+                )
 
         def volumeRemoved(self, volumeMonitor, volume: Gio.Volume) -> None:
             logging.debug("GIO: %s volume removed", volume.get_name())
@@ -1465,7 +1636,7 @@ if have_gio:
 
 
 def _get_info_size_value(info: Gio.FileInfo, attr: str) -> int:
-    if info.get_attribute_data(attr).type ==  Gio.FileAttributeType.UINT64:
+    if info.get_attribute_data(attr).type == Gio.FileAttributeType.UINT64:
         return info.get_attribute_uint64(attr)
     else:
         return info.get_attribute_uint32(attr)
@@ -1490,8 +1661,11 @@ def get_mount_size(mount: QStorageInfo) -> Tuple[int, int]:
 
     logging.debug("Using GIO to query file system attributes for %s...", path)
     p = Gio.File.new_for_path(os.path.abspath(path))
-    info = p.query_filesystem_info(','.join((Gio.FILE_ATTRIBUTE_FILESYSTEM_SIZE,
-                                             Gio.FILE_ATTRIBUTE_FILESYSTEM_FREE)))
+    info = p.query_filesystem_info(
+        ','.join(
+            (Gio.FILE_ATTRIBUTE_FILESYSTEM_SIZE, Gio.FILE_ATTRIBUTE_FILESYSTEM_FREE)
+        )
+    )
     logging.debug("...query of file system attributes for %s completed", path)
     bytes_total = _get_info_size_value(info, Gio.FILE_ATTRIBUTE_FILESYSTEM_SIZE)
     bytes_free = _get_info_size_value(info, Gio.FILE_ATTRIBUTE_FILESYSTEM_FREE)
